package json_sample_project

import (
	"github.com/goccy/go-json"
	"github.com/marcsantiago/gojay"
	"json_sample_project/gojaywrappers"
	"json_sample_project/optional"
)

type Banner struct {
	ID       string                     `json:"id,omitempty"       `
	BidFloor optional.Optional[float64] `json:"bidfloor,omitempty"   `
	BAttr    []int                      `json:"battr,omitempty"    `
	W        int                        `json:"w,omitempty"        `
	H        int                        `json:"h,omitempty"        `
	Pos      int                        `json:"pos,omitempty"      `
	Mimes    []string                   `json:"mimes,omitempty"    `
	API      []int                      `json:"api,omitempty"      `
	Vcm      int                        `json:"vcm,omitempty"      `
	Ext      *BannerExt                 `json:"ext,omitempty"      `
}

// BannerExt ...
type BannerExt struct {
	json.Marshaler            `json:"-"`
	gojay.MarshalerJSONObject `json:"-"`
}

func (b *Banner) MarshalJSONObject(enc *gojay.Encoder) {
	enc.StringKeyOmitEmpty("id", b.ID)
	if b.BidFloor.IsSome() {
		enc.Float64KeyOmitEmpty("bidfloor", b.BidFloor.Get())
	}
	var battrSlice = gojaywrappers.Ints(b.BAttr)
	enc.ArrayKeyOmitEmpty("battr", battrSlice)
	enc.IntKeyOmitEmpty("w", b.W)
	enc.IntKeyOmitEmpty("h", b.H)
	enc.IntKeyOmitEmpty("pos", b.Pos)
	var mimesSlice = gojaywrappers.Strings(b.Mimes)
	enc.ArrayKeyOmitEmpty("mimes", mimesSlice)
	var aPISlice = gojaywrappers.Ints(b.API)
	enc.ArrayKeyOmitEmpty("api", aPISlice)
	enc.IntKeyOmitEmpty("vcm", b.Vcm)
	enc.ObjectKeyOmitEmpty("ext", b.Ext)
}

func (b *Banner) IsNil() bool {
	return b == nil
}

func (b *Banner) UnmarshalJSONObject(dec *gojay.Decoder, k string) error {

	switch k {
	case "id":
		return dec.String(&b.ID)

	case "bidfloor":
		var value float64
		err := dec.Float64(&value)
		if err == nil {
			b.BidFloor.Set(value)
		}
		return err

	case "battr":
		var aSlice = gojaywrappers.Ints{}
		err := dec.Array(&aSlice)
		if err == nil && len(aSlice) > 0 {
			b.BAttr = aSlice
		}
		return err

	case "w":
		return dec.Int(&b.W)

	case "h":
		return dec.Int(&b.H)

	case "pos":
		return dec.Int(&b.Pos)

	case "mimes":
		var aSlice = gojaywrappers.Strings{}
		err := dec.Array(&aSlice)
		if err == nil && len(aSlice) > 0 {
			b.Mimes = aSlice
		}
		return err

	case "api":
		var aSlice = gojaywrappers.Ints{}
		err := dec.Array(&aSlice)
		if err == nil && len(aSlice) > 0 {
			b.API = aSlice
		}
		return err

	case "vcm":
		return dec.Int(&b.Vcm)

	}
	return nil
}

func (b *Banner) NKeys() int { return 0 }

type Banners []Banner

func (s *Banners) UnmarshalJSONArray(dec *gojay.Decoder) error {
	var value = Banner{}
	if err := dec.Object(&value); err != nil {
		return err
	}
	*s = append(*s, value)
	return nil
}

func (s Banners) MarshalJSONArray(enc *gojay.Encoder) {
	for i := range s {
		enc.Object(&s[i])
	}
}

func (s Banners) IsNil() bool {
	return len(s) == 0
}

func (e *BannerExt) MarshalJSONObject(enc *gojay.Encoder) {
	if e.MarshalerJSONObject != nil {
		e.MarshalerJSONObject.MarshalJSONObject(enc)
	}
}

func (e *BannerExt) IsNil() bool {
	return e == nil
}
