package json_sample_project

import (
	stdJson "encoding/json"
	"github.com/goccy/go-json"
	"github.com/marcsantiago/gojay"
	"json_sample_project/optional"
	"testing"
)

var expected = `{"id":"1","bidfloor":0.1,"battr":[1,3],"w":300,"h":250,"pos":1,"mimes":["image/jpeg","image/gif"],"api":[3],"ext":{"tag":"banner"}}`

func TestGoJay(t *testing.T) {
	banner := CreateBanner(true)
	b, err := gojay.MarshalJSONObject(banner)
	if err != nil {
		t.Fatal(err)
	}
	if string(b) != expected {
		t.Fatalf("unexpected result: %s", string(b))
	}
}

func TestGoJson(t *testing.T) {
	banner := CreateBanner(false)
	b, err := json.Marshal(banner)
	if err != nil {
		t.Fatal(err)
	}
	if string(b) != expected {
		t.Fatalf("unexpected result: %s", string(b))
	}
}

func CreateBanner(useGoJayExt bool) *Banner {
	banner := &Banner{
		W:        300,
		H:        250,
		ID:       "1",
		BidFloor: optional.New(0.1),
		BAttr: []int{
			1,
			3,
		},
		Pos: 1,
		Mimes: []string{
			"image/jpeg",
			"image/gif",
		},
		API: []int{
			3,
		},
	}
	if useGoJayExt {
		banner.Ext = &BannerExt{
			Marshaler: nil,
			MarshalerJSONObject: &extJay{
				Tag: "banner",
			},
		}
	} else {
		banner.Ext = &BannerExt{
			Marshaler: &extJay{
				Tag: "banner",
			},
			MarshalerJSONObject: nil,
		}
	}

	return banner
}

type extJay struct {
	Tag string `json:"tag,omitempty"`
}

func (e *extJay) MarshalJSON() ([]byte, error) {
	if e == nil {
		return []byte("null"), nil
	}

	type Alias extJay
	return json.Marshal(&struct{ *Alias }{Alias: (*Alias)(e)})
}

func (e *extJay) MarshalJSONObject(enc *gojay.Encoder) {
	enc.StringKeyOmitEmpty("tag", e.Tag)
}

func (e *extJay) IsNil() bool {
	return e == nil
}

func BenchmarkGoJay(b *testing.B) {
	banner := CreateBanner(true)
	for i := 0; i < b.N; i++ {
		_, err := gojay.MarshalJSONObject(banner)
		if err != nil {
			b.Fatal(err)
		}
	}
}

func BenchmarkGoJson(b *testing.B) {
	banner := CreateBanner(false)
	for i := 0; i < b.N; i++ {
		_, err := json.Marshal(banner)
		if err != nil {
			b.Fatal(err)
		}
	}
}

func BenchmarkStdJson(b *testing.B) {
	banner := CreateBanner(false)
	for i := 0; i < b.N; i++ {
		_, err := stdJson.Marshal(banner)
		if err != nil {
			b.Fatal(err)
		}
	}
}
